#pragma once

class Point3
{
public:
    union
	{
        struct
		{
            int x, y, z;
        };
        int val[3];
    };

	Point3();
	Point3(int x, int y, int z);
    inline bool operator == ( const Point3& r ) const
    {
        return ( x == r.x && y == r.y && z == r.z );
    }
	
    inline Point3& operator = ( const Point3& rkVector )
    {
        x = rkVector.x;
        y = rkVector.y;
        z = rkVector.z;

        return *this;
    }

    inline bool operator != ( const Point3& rkVector ) const
    {
        return ( x != rkVector.x || y != rkVector.y || z != rkVector.z );
    }

    // arithmetic operations
    inline Point3 operator * ( const int rkVector ) const
    {
        Point3 kSum;

        kSum.x = x * rkVector;
        kSum.y = y * rkVector;
        kSum.z = z * rkVector;

        return kSum;
    }

    // arithmetic operations
    inline Point3 operator + ( const Point3& rkVector ) const
    {
        Point3 kSum;

        kSum.x = x + rkVector.x;
        kSum.y = y + rkVector.y;
        kSum.z = z + rkVector.z;

        return kSum;
    }

    inline Point3 operator - ( const Point3& rkVector ) const
    {
        Point3 kDiff;

        kDiff.x = x - rkVector.x;
        kDiff.y = y - rkVector.y;
        kDiff.z = z - rkVector.z;

        return kDiff;
    }

    // arithmetic updates
    inline Point3& operator += ( const Point3& rkVector )
    {
        x += rkVector.x;
        y += rkVector.y;
        z += rkVector.z;

        return *this;
    }

    inline Point3& operator -= ( const Point3& rkVector )
    {
        x -= rkVector.x;
        y -= rkVector.y;
        z -= rkVector.z;

        return *this;
    }

	inline bool operator< (const Point3 &b) const
	{
		if (x < b.x)
		  return true;

		if (b.x < x)
		  return false;

		// now, we know that a.x == b.x
		// compare on y

		if (y < b.y)
		  return true;

		if (b.y < y)
		  return false;

		// now, we know that (a.x == b.x) && (a.y == b.y)
		// compare on y

		if (z < b.z)
		  return true;

		return false;
	}
};

class Point3Compare
{
public:
  bool operator()(const Point3 & a, const Point3 & b) const
  {
    if (a.x < b.x)
      return true;

    if (b.x < a.x)
      return false;

    // now, we know that a.x == b.x
    // compare on y

    if (a.y < b.y)
      return true;

    if (b.y < a.y)
      return false;

    // now, we know that (a.x == b.x) && (a.y == b.y)
    // compare on y

    if (a.z < b.z)
      return true;

    return false;

  }
};
