#include "stdafx.h"
#include "Cell.h"
#include "Particle.h"
#include "Body.h"

Point3 vertexOffset[8] = {Point3(0,0,0), Point3(0,0,1), Point3(0,1,0), Point3(0,1,1), Point3(1,0,0), Point3(1,0,1), Point3(1,1,0), Point3(1,1,1)};

Cell::Cell()
{
}

void Cell::Initialize()
{
	Point3 p2 = center->index*2;

	for(unsigned int i = 0; i < 8; i++)
	{
		CellVertex *v = &vertices[i];

		v->owner = this;
		Vector3f &spacing = center->body->spacing;
		v->materialPosition = center->particle->x0 + Vector3f(spacing.x * ((float)vertexOffset[i].x - 0.5f), spacing.y * ((float)vertexOffset[i].y - 0.5f), spacing.z * ((float)vertexOffset[i].z - 0.5f));

		// Set up the vertex's shareVertexCells
		// The vertex's index
		Point3 v2 = p2 + vertexOffset[i]*2 - Point3(1,1,1);
		v->v2 = v2;
		for(unsigned int j = 0; j < 8; j++)
		{
			// The shared point's offset
			Point3 s2 = v2 + vertexOffset[j]*2 - Point3(1,1,1);
			Point3 s = Point3(s2.x / 2, s2.y / 2, s2.z / 2);

			if(center->body->GetLatticeLocation(s) != NULL)
				v->shareVertexCells[j] = (Cell*)center->body->GetLatticeLocation(s)->cell;
		}
	}
	
	// Set up the connected cells
	for each(LatticeLocation *lp in center->immediateNeighbors)
	{
		connectedCells.push_back(lp->cell);
	}
	connectedCells.push_back(this);		// I'm always connected to myself
}

void Cell::Initialize2()
{
	// For each vertex
	for(unsigned int i = 0; i < 8; i++)
	{
		CellVertex *v = &vertices[i];

		// Set up the partnerVertices
		// For each shareVertexCell
		for(unsigned int j = 0; j < 8; j++)
		{
			Cell *shared = v->shareVertexCells[j];
			if(shared != NULL)
			{
				// For each of the vertices of the shareVertexCell
				for(unsigned int l = 0; l < 8; l++)
				{
					if(shared->vertices[l].v2 == v->v2)//shared->vertices[l].materialPosition == v->materialPosition)
					{
						// That is our shared buddy
						v->partnerVertices[j] = &shared->vertices[l];
					}
				}
				assert(v->partnerVertices[j]);
			}
		}

		v->DeterminePositionArbiter();
		assert(v->positionArbiter);
	}
}

void Cell::HandleVertexSharerFracture()
{
	for(unsigned int i = 0; i < 8; i++)
	{
		vertices[i].HandleVertexSharerFracture();
	}
}

void Cell::UpdateVertexPositions()
{
	for(int i = 0; i < 8; i++)
	{
		vertices[i].UpdatePosition();
	}
}
